"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const util_1 = require("util");
const _l = require("lodash");
const findup = require("findup");
const utils_1 = require("./utils");
function loadPagedrawConfig() {
    return __awaiter(this, void 0, void 0, function* () {
        let dir;
        try {
            dir = yield util_1.promisify(findup)(process.cwd(), 'pagedraw.json');
        }
        catch (err) {
            throw new utils_1.CLIPrintableError("Unable to find pagedraw.json in ancestor directories.");
        }
        var config;
        try {
            config = JSON.parse(fs.readFileSync(path.join(dir, 'pagedraw.json'), 'utf8'));
        }
        catch (err) {
            throw new utils_1.CLIPrintableError("Error reading pagedraw.json. Is it a properly formatted json file?");
        }
        if (!_l.isString(config.app)) {
            throw new utils_1.CLIPrintableError("pagedraw.json must specify an \"app\".  It's expected to be a string.");
        }
        let managed_folders_typechecks = !('managed_folders' in config)
            || (_l.isArray(config.managed_folders) && _l.every(config.managed_folders, (f) => _l.isString(f)));
        if (!managed_folders_typechecks) {
            throw new utils_1.CLIPrintableError("pagedraw.json's \"managed_folders\" must be an array of strings.");
        }
        process.chdir(dir);
        return config;
    });
}
exports.loadPagedrawConfig = loadPagedrawConfig;
//# sourceMappingURL=config.js.map