"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const netrc = require("netrc");
const _l = require("lodash");
const utils_1 = require("./utils");
class CredentialStore {
    constructor(netrc_entry) {
        this.netrc_entry = netrc_entry;
        this.cachedCredentials = null;
    }
    get credentials() {
        if (this.cachedCredentials == null) {
            this.cachedCredentials = this.uncached_read();
        }
        if (this.cachedCredentials == null) {
            return utils_1.abort('User is not authenticated to access the Pagedraw API. Please run pagedraw login');
        }
        return this.cachedCredentials;
    }
    uncached_read() {
        var netrcCreds = netrc()[this.netrc_entry];
        if (_l.isEmpty(netrcCreds) || _l.isEmpty(netrcCreds.login) || _l.isEmpty(netrcCreds.password)) {
            return null;
        }
        return { id: netrcCreds.login, auth_token: netrcCreds.password };
    }
    persist(credentials) {
        var myNetrc = netrc();
        myNetrc[this.netrc_entry] = { login: credentials.id, password: credentials.auth_token };
        netrc.save(myNetrc);
        this.cachedCredentials = _l.clone(credentials);
    }
}
exports.CredentialStore = CredentialStore;
//# sourceMappingURL=credentials.js.map