"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _l = require("lodash");
const url = require("url");
const request = require("request");
const utils_1 = require("./utils");
const pdAPI = require("./api");
const pkgJson = require('../package.json');
function checkPackageInfo(info) {
    if (_l.isEmpty(info)) {
        return utils_1.abort('Unable to fetch CLI info from server. Please report this error.');
    }
    if (info.version != pkgJson.version || info.name != pkgJson.name) {
        return utils_1.abort(`Your Pagedraw CLI is out of date. Please run\n\tnpm install -g ${info.name}@${info.version}`);
    }
}
;
function enforceClientUpToDate(continuation) {
    if (process.env['ENVIRONMENT'] == 'development') {
        return continuation();
    }
    utils_1.log('Getting CLI package info');
    onceCLIInfo((err, info) => {
        if (err && err.code == 'ENOTFOUND') {
            return utils_1.abort('Unable to verify that the Pagedraw CLI is up to date. Are you connected to the internet?');
        }
        if (err || info === undefined) {
            throw err;
        }
        checkPackageInfo(info);
        return continuation();
    });
}
exports.enforceClientUpToDate = enforceClientUpToDate;
;
function long_term_enforce_client_up_to_date(callback) {
    enforceClientUpToDate(() => {
        callback();
        if (process.env['ENVIRONMENT'] != 'development') {
            watchCLIInfo(checkPackageInfo);
        }
    });
}
exports.long_term_enforce_client_up_to_date = long_term_enforce_client_up_to_date;
function onceCLIInfo(callback) {
    request.get(url.resolve(pdAPI.DOCSERVER, 'cli_info.json'), (err, resp, body) => {
        return err ?
            callback(err) :
            callback(null, JSON.parse(body));
    });
}
function watchCLIInfo(callback) {
    const ref = pdAPI.firebaseDB().ref(`cli_info`);
    const watch_id = ref.on('value', ((info) => callback(info && info.val())), ((error) => { throw error; }));
}
//# sourceMappingURL=keep_up_to_date.js.map