"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const _l = require("lodash");
const fs = require("fs");
const path = require("path");
const filendir = require("filendir");
const utils = require("./utils");
const pdAPI = require("./api");
const clc = require("cli-color");
const util_1 = require("util");
const readdirRecursive = require("fs-readdir-recursive");
const doc_link = (id) => `https://pagedraw.io/pages/${id}`;
function pullPagedrawDoc(doc, docsAllowedToTouch, managed_folders) {
    return __awaiter(this, void 0, void 0, function* () {
        utils.assert(() => docsAllowedToTouch.includes(doc));
        let { files, messages } = yield pdAPI.compileFromDocserverId(doc.docserver_id);
        for (let message of messages) {
            if (message.level === "error") {
                console.error(clc.red(`[${doc.url}] ${message.message}`));
            }
            else if (message.level === "warning") {
                console.log(clc.yellow(`[${doc.url}] ${message.message}`));
            }
            else {
                const _exhaustiveCheck = message.level;
            }
        }
        if (files === null) {
            return;
        }
        if (_l.isEmpty(files)) {
            console.log(clc.yellow(`[${doc.url}] No components to pull. Please ensure this doc has components marked "Should pull/sync from CLI" in the editor.`));
        }
        const blacklist = _l.map(files, (r) => path.resolve(r.filePath));
        yield removeFilesOwnedByDocs([doc], managed_folders, blacklist);
        try {
            yield Promise.all(_l.map(files, (result) => __awaiter(this, void 0, void 0, function* () {
                let page_id;
                try {
                    page_id = yield readPagedrawnFileHeader(result.filePath);
                }
                catch (err) {
                    if (err.code == 'ENOENT') {
                    }
                    else if (err.code == 'NON-PAGEDRAWN') {
                        console.error(clc.red(`[${doc.url}] ${result.filePath}: Found non-Pagedraw file in this path. Not overwriting.`));
                        return;
                    }
                    else {
                        console.error(clc.red(`[${doc.url}] error reading ${result.filePath}: ${err.code}`));
                        return;
                    }
                }
                if (page_id && !_l.find(docsAllowedToTouch, (doc) => doc.id == page_id)) {
                    console.log(clc.yellow(`[${doc.url}] ${result.filePath}: Pagedraw doc ${page_id} (${doc_link(page_id)}) already has a component in this path. Not overwriting.`));
                    return;
                }
                if (!canWriteToFilePathSafely(result.filePath)) {
                    console.error(clc.red(`[${doc.url}] ${result.filePath}: File path outside Pagedraw controlled path. Not writing.`));
                    return;
                }
                else {
                    filendir.writeFile(result.filePath, result.contents, (err) => {
                        if (err && err.code == 'ENOENT') {
                            console.error(clc.red(`[${doc.url}] Failed to create file at ${result.filePath}. Are you trying to write to a directory that does not exist?`));
                            return;
                        }
                        if (err) {
                            return utils.abort(err.message);
                        }
                        console.log(clc.green(`[${doc.url}] Synced at path ${result.filePath}`));
                    });
                }
            })));
        }
        catch (e) {
            throw (e);
        }
    });
}
exports.pullPagedrawDoc = pullPagedrawDoc;
function removeFilesOwnedByDocs(docs, managed_folders, blacklist) {
    return __awaiter(this, void 0, void 0, function* () {
        yield Promise.all(managed_folders.map((folder) => __awaiter(this, void 0, void 0, function* () {
            let files;
            try {
                files = readdirRecursive(folder);
            }
            catch (err) {
                return;
            }
            yield Promise.all(files.map((filename) => __awaiter(this, void 0, void 0, function* () {
                const filepath = path.resolve(folder, filename);
                if (blacklist.includes(filepath)) {
                    return;
                }
                let page_id;
                try {
                    page_id = readPagedrawnFileHeader(filepath);
                }
                catch (err) {
                    return;
                }
                if (_l.find(docs, (doc) => doc.id == page_id)) {
                    yield util_1.promisify(fs.unlink)(filepath);
                }
            })));
        })));
    });
}
const isInsideDir = (filepath, parent_dir) => {
    const relative = path.relative(parent_dir, filepath);
    return !!relative && !relative.startsWith('..') && !path.isAbsolute(relative);
};
function canWriteToFilePathSafely(filepath) {
    let pathToWriteTo = path.resolve(process.cwd(), filepath);
    return isInsideDir(pathToWriteTo, process.cwd());
}
function readPagedrawnFileHeader(filepath) {
    return __awaiter(this, void 0, void 0, function* () {
        let data = yield util_1.promisify(fs.readFile)(filepath, "utf8");
        const match = data.match(/Generated by https:\/\/pagedraw.io\/pages\/(\d+)/);
        if (match == null) {
            throw { code: 'NON-PAGEDRAWN' };
        }
        return match[1];
    });
}
//# sourceMappingURL=pd-syncer.js.map