"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const clc = require("cli-color");
const VERBOSE = process.env['VERBOSE'] || false;
function abort(message, error_code = 1) {
    console.error(clc.red(message));
    return process.exit(error_code);
}
exports.abort = abort;
function log(message) {
    if (VERBOSE) {
        console.log('<log> ' + message);
    }
}
exports.log = log;
function assert(fn) {
    if (!fn()) {
        abort('Assertion failed');
    }
}
exports.assert = assert;
function poll(ms, fn) {
    fn(() => {
        setTimeout((() => { poll(ms, fn); }), ms);
    });
}
exports.poll = poll;
function poll_with_max_retries(ms, max_retries, fn, fail) {
    poll(ms, (retry) => {
        fn(() => {
            max_retries -= 1;
            if (max_retries > 0) {
                retry();
            }
            else {
                fail();
            }
        });
    });
}
exports.poll_with_max_retries = poll_with_max_retries;
class CLIPrintableError {
    constructor(message) {
        this.message = message;
    }
    ;
}
exports.CLIPrintableError = CLIPrintableError;
//# sourceMappingURL=utils.js.map